import logging
import time
import json


class Logger(object):
    levels = {
        'DEBUG': 100,
        'INFO': 200,
        'WARN': 300,
        'ERROR': 400,
    }
    time_format = '%y-%m-%d %H:%M'

    def __init__(self, obj='root', level='DEBUG'):
        self._level = level
        self._levels = Logger.levels
        self._message = obj.__repr__()
        self._message_bak = obj.__str__()
        self._time_format = Logger.time_format

    @property
    def level(self):
        return self._level

    @level.setter
    def level(self, level):
        if level in self._levels:
            self._level = level
        else:
            raise ValueError

    @property
    def time_fmt(self):
        return self._time_format

    @time_fmt.setter
    def time_fmt(self, fmt):
        self._time_format = fmt

    def time(self):
        self._message += ' ' + time.strftime(self._time_format)
        return self

    def json(self, dictionary):
        self._message += '\n' + json.dumps(dictionary, indent=2, ensure_ascii=False)
        return self

    def message(self, message):
        self._message += '\n' + message.__str__()
        return self

    def print_message(self, message, level):
        if Logger.levels.get(level) >= Logger.levels.get(self._level):
            self._message = level + ': ' + self._message + ' '
        print(self._message,)
        print('\t', message)
        self._message = self._message_bak

    def info(self, message):
        return self.print_message(message, 'INFO')

    def debug(self, message):
        return self.print_message(message, 'DEBUG')

    def warn(self, message):
        return self.print_message(message, 'WARN')

    def error(self, message):
        return self.print_message(message, 'ERROR')



import csv
import os
import sqlite3

from zoran_tools.path_tools import create_father_dir


def readcsv(filename, encoding='utf8', delimiter=',', quotechar='"', li=True):
    """
    接收一个CSV文件名, 返回其内容, 可能返回<list>也可能返回<generator>
    :param filename: <str> 要读取的CSV路径
    :param encoding:
    :param delimiter:
    :param quotechar:
    :param li: True or False, 指定返回的数据格式, 为True时返回列表, 为False时返回生成器
    :return:
    """
    with open(file=filename, mode='r', encoding=encoding, newline='') as f:
        f_csv = csv.reader(f, delimiter=delimiter, quotechar=quotechar)
        if li:
            return [row for row in f_csv]
        else:
            for row in f_csv:
                yield row


def write_csv(filename, row_or_rows, mode='a+', encoding='utf8', newline='',
              delimiter=',', quotechar='"', quoting='all'):
    """
    接收一个文件名(路径)和一个列表, 将列表写入该文件
    :param filename: <str> 要写入的文件名
    :param row_or_rows: <list> 要写入的列表
    :param mode:
    :param encoding:
    :param newline:
    :param delimiter:
    :param quotechar:
    :param quoting: <csv.QUOTE_MINIMAL, csv.QUOTE_ALL, csv.QUOTE_NONNUMERIC, csv.QUOTE_NONE>
        csv.QUOTE_MINIMAL 仅当字段中包含分隔符时才用引号括起,
        csv.QUOTE_ALL 任何情况下都将字段用引号括起,
        csv.QUOTE_NONNUMERIC 括起非数字字段, 数字不括起,
        csv.QUOTE_NONE 不括起任何字段
    """
    create_father_dir(filename)

    quoting_dict = {
        'minimal': csv.QUOTE_MINIMAL,
        'nonnumeric': csv.QUOTE_NONNUMERIC,
        'all': csv.QUOTE_ALL,
        'none': csv.QUOTE_NONE,
    }
    quoting = quoting_dict.get(quoting.lower(), csv.QUOTE_NONE)

    with open(file=filename, mode=mode, encoding=encoding, newline=newline) as f:
        sp = csv.writer(f, delimiter=delimiter, quotechar=quotechar, quoting=quoting)
        if all([isinstance(e, (list, tuple)) for e in row_or_rows]):
            sp.writerows(row_or_rows)
        else:
            sp.writerow(row_or_rows)


def write_csv_row(filename, row, mode='a+', encoding='utf8', newline='',
              delimiter=',', quotechar='"', quoting='all'):
    return write_csv(filename, row, mode, encoding, newline, delimiter, quotechar, quoting)


def write_csv_rows(filename, rows, mode='a+', encoding='utf8', newline='',
              delimiter=',', quotechar='"', quoting='all'):
    return write_csv(filename, rows, mode, encoding, newline, delimiter, quotechar, quoting)





import re
import json


def jsonp_to_json(jsonp_text, format='str'):
    """
    将Jsonp格式以Json格式解析出来
    :param jsonp_text: 要解析的Jsonp文本
    :param format: 要解析为的格式, 可以是str或dict
    :return:
    """
    text_wihtout_new_line = re.sub('\s', '', jsonp_text)
    jsn = re.findall('.*\(({.*})\)[;]{0,1}', text_wihtout_new_line)[0]
    jsn_dict = json.loads(jsn)
    if format == 'dict':
        return jsn_dict
    elif format == 'str':
        return json.dumps(jsn_dict, indent=2, ensure_ascii=False)
    else:
        raise ValueError


def _plot(node, plot='', level=0):
    if isinstance(node, str):
        plot += (' ' * 2) * (level + 1) + '|-- ' + node + '\n'
    elif isinstance(node, dict):
        for k in node:
            v = node[k]
            plot = _plot(plot=plot, node=k, level=level)
            plot = _plot(plot=plot, node=v, level=level + 1)
    elif isinstance(node, (list, tuple)):
        node = sorted(node, key=lambda e: not isinstance(e, str))
        for i, e in enumerate(node):
            plot = _plot(plot=plot, node=e, level=level + 1)
    else:
        plot = _plot(plot=plot, node=node.__str__(), level=level + 1)

    return plot


def plot_json_tree(d):
    print(_plot(d))



"""
模块的主要功能是进行路径操作
"""

import os
import tkinter as tk
from tkinter import filedialog
from pathlib import Path


class ZPath(object):
    def __init__(self, path):
        self._path = path

    def __str__(self):
        return '<zoran_tools.Path>\n{}'.format(self.path)

    def __repr__(self):
        return self.__str__()

    @property
    def path(self):
        return self._path

    @path.setter
    def path(self, path):
        self._path = path

    @property
    def basename(self):
        return os.path.basename(self.path)

    @property
    def abspath(self):
        return os.path.abspath(self.path)

    @property
    def dirname(self):
        return os.path.dirname(self.abspath)

    def join(self, *paths, split=True, new=False):
        if split:
            dir_name = os.path.splitext(self.abspath)[0]
        else:
            dir_name = self.path
        if new:
            return ZPath(os.path.join(dir_name, *paths))
        else:
            return os.path.join(dir_name, *paths)

    def make(self, split=True, new=False):
        if split:
            dir_name = os.path.splitext(self.abspath)[0]
        else:
            dir_name = self.path
        if not os.path.exists(dir_name) or not os.path.isdir(dir_name):
            os.makedirs(dir_name)
        if new:
            return ZPath(dir_name)
        else:
            return dir_name

    def isdir(self):
        return os.path.isdir(self.path)

    def isfile(self):
        return os.path.isfile(self.path)

    def exist(self):
        return os.path.exists(self.abspath)

    def children(self):
        return {self.basename: os.listdir(self.path)}

    def tree(self, maxlevel=3):
        if maxlevel == 0:
            return self.basename
        try:
            return {
                self.basename: [
                    self.join(e, new=True).tree(maxlevel=maxlevel-1)
                    if self.join(e, new=True).isdir() else self.join(e, new=True).basename
                    for e in os.listdir(self.path)
                ]
            }
        except PermissionError:
            return self.basename

    def plot_tree(self, maxlevel=3):
        def _plot(node, plot='', level=0):
            if isinstance(node, str):
                plot += (' ' * 2) * (level + 1) + '|-- ' + node + '\n'
            elif isinstance(node, dict):
                for k in node:
                    v = node[k]
                    plot = _plot(plot=plot, node=k, level=level)
                    plot = _plot(plot=plot, node=v, level=level + 1)
            elif isinstance(node, list):
                node = sorted(node, key=lambda e: not isinstance(e, str))
                for i, e in enumerate(node):
                    plot = _plot(plot=plot, node=e, level=level + 1)
            return plot

        return _plot(node=self.tree(maxlevel=maxlevel))


class File(ZPath):
    pass


class Directory(ZPath):
    pass


def plot_tree(path=None, maxlevel=3):
    if path is None:
        path = os.getcwd()
    return ZPath(path).plot_tree(maxlevel=maxlevel)


def list_files(directory=None, fm=None, return_abs=False, mode=None):
    """
    返回文件夹下的所有文件<list>
    :param directory: <str> 文件夹路径
        如果给出文件夹路径, 则返回该文件夹下的文件;
        如果没有给出文件夹路径, 则返回控制台所在文件夹下的文件
    :param fm: <str, list> 指定文件格式
        如果给出了文件格式, 则返回指定格式的文件;
        如果没有给出文件格式, 则返回所有文件
    :param isabspath: <bool> 为真时返回绝对路径文件名, 为假时返回相对路径文件名
    """
    if not directory:
        directory = os.getcwd()
    files = os.listdir(directory)

    if isinstance(mode, File):
        files = [file for file in files if os.path.isfile(file)]
    elif isinstance(mode, Directory):
        files = [file for file in files if os.path.isdir(file)]
    else:
        pass
    
    if isinstance(fm, str):
        fm = [fm]
    if isinstance(fm, (list, tuple)):
        fm = ['.{}'.format(e) for e in fm]
    files = [file for file in files if os.path.splitext(file)[-1] in fm]

    if return_abs:
        files = [os.path.abspath(file) for file in files]
    else:
        files = [os.path.basename(file) for file in files]

    return files


def create_father_dir(filename):
    """
    接收一个文件名, 判断其父路径是否存在, 如果不存在, 则创建
    :param filename: <str>接收的文件路径, 为相对路径
    """
    abs_filename = os.path.abspath(filename)
    father_dir = os.path.dirname(abs_filename)
    if not os.path.exists(father_dir) or not os.path.isdir(father_dir):
        os.makedirs(father_dir)
    return father_dir


def create_dir_same_as_filename(filename):
    """
    创建文件同名文件夹
    :param filename: <str>接收的文件路径, 为相对路径或绝对路径
    :param tail: <str>有时候生成的文件夹后要加个尾缀
    """
    abs_filename = os.path.abspath(filename)
    split_extension_abs_filename = os.path.splitext(abs_filename)[0]
    if os.path.exists(split_extension_abs_filename) and os.path.isdir(split_extension_abs_filename):
        return split_extension_abs_filename
    else:
        os.makedirs(split_extension_abs_filename)
        return split_extension_abs_filename


# tkinter.filedialog.asksaveasfile():选择以什么文件保存，创建文件并返回文件流对象
# tkinter.filedialog.askopenfile():选择打开什么文件，返回IO流对象
# tkinter.filedialog.askopenfiles():选择打开多个文件，以列表形式返回多个IO流对象
def get_goal_by_dialog_box(goal='file', filetype=None):
    """
    启用对话框, 以根据goal参数的不同选择文件或文件夹
    :param goal:
    :param filetype:
    :return: 返回文件名或文件夹名
    """
    root = tk.Tk()
    root.withdraw()

    goal_dict = {
        'file': filedialog.askopenfilename,
        'files': filedialog.askopenfilenames,
        'directory': filedialog.askdirectory,
        'dir': filedialog.askdirectory,
        'saveas': filedialog.asksaveasfilename,
        'save_as': filedialog.asksaveasfilename,
    }

    goal_func = goal_dict.get(goal)
    goal_name = goal_func(filetype=filetype) if isinstance(filetype, tuple) else goal_func()

    root.destroy()
    return goal_name


def ask_file(filetype=None):
    """
    打开一个对话框, 以选择文件, 返回文件路径.
    利用了tkinter框架
    示例：
    a = ask_file(filetype=(
        ('text file', '*.md *.txt'),
        ('word file', '*.doc *.docx'),
        ('all file', '*'),
        )
    )
    :return: 返回文件绝对路径名
    """
    return get_goal_by_dialog_box(goal='file', filetype=filetype)


def ask_files(filetype=None):
    """
    打开一个对话框, 以选择多个文件, 返回文件名列表
    :return:
    """
    return get_goal_by_dialog_box(goal='files', filetype=filetype)


def ask_dir():
    """
    打开一个对话框, 以选择文件夹, 返回文件夹名
    :return:
    """
    return get_goal_by_dialog_box(goal='directory')


def ask_save_as():
    """
    打开一个对话框，以选择文件名，如果文件名已存在，则询问是否覆盖
    :return:
    """
    return get_goal_by_dialog_box(goal='save_as')


def ask_chdir():
    """
    切换控制台路径
    :return:
    """
    return os.chdir(ask_dir())


def get_size(dir_or_file, unit='m'):
    """
    计算文件或文件夹的大小
    :param dir_or_file: 文件或文件夹路径
    :param unit: 返回的单位，默认为MB
    :return:
    """
    size = 0

    if os.path.isfile(dir_or_file):
        size = os.path.getsize(dir_or_file)
    elif os.path.isdir(dir_or_file):
        for root, dirs, files in os.walk(dir_or_file):
            size += sum(map(lambda e: os.stat(os.path.join(root, e)).st_size, files))
    else:
        return 0

    unit_dict = {
        'k': lambda x: x / 1024,
        'kb': lambda x: x / 1024,
        'm': lambda x: x / 1024 / 1024,
        'mb': lambda x: x / 1024 / 1024,
        'g': lambda x: x / 1024 / 1024 / 1024,
        'gb': lambda x: x / 1024 / 1024 / 1024,
    }

    return '{:.2f}'.format(unit_dict.get(unit.lower(), lambda x: x)(size))




import sys
import math
import codecs


def readfile(file, encoding='utf8'):
    """
    读取文本文件, 返回文本文件内容
    :param file: 要读取的文件名
    :param encoding: 文件编码, 默认为utf8
    :return: 返回文件内容
    """
    with codecs.open(file, mode='r', encoding=encoding) as f:
        return f.read()


def writefile(file, content, mode='w', encoding='utf8'):
    """
    将文本内容写入文件
    :param file: 要写入的文件名
    :param content: 要写入的内容
    :param mode: 写入模式, 默认为w覆盖
    :param encoding: 写入编码, 默认为utf8
    :return:
    """
    with codecs.open(file, mode=mode, encoding=encoding) as f:
        f.write(content)


def transcode(from_file, to_file, from_code='utf8', to_code='GBK'):
    """
    转换文本文件格式
    :param from_file: 待转换的文件名
    :param to_file: 转换后的文件名
    :param from_code: 转换前的文件编码
    :param to_code: 转换后的文件编码
    :return:
    """
    content = readfile(from_file, encoding=from_code)
    writefile(to_file, content, encoding=to_code)
    return '{} ====> {}'.format(from_file, to_file)


def split_list(li, num=8):
    """
    分割列表
    :param li: 要分割的列表
    :param num: 要分割的份数
    :return: 返回分割结果
    """
    return split_list_by_len(li, math.ceil(len(li) / num))


def split_list_by_len(li, n):
    return [li[i: i + n] for i in range(0, len(li), n)]


class WithProgressBar(object):
    """
    用于在控制台打印一个进度条，用法（作用于for 循环）：
        # 仅打印进度条
        for i in WithProgressBar(range(1000)):
           pass
        ##############################################----98%

        # 打印进度条，并获得一部分预览数据
        preview_data = PreviewDataInBar()
        for i in WithProgressBar(range(1000), preview_data=preview_data):
            pass
        ##################################################100%
        print(preview_data.preview)
        [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11,... ...199]
    """
    def __init__(self, iterable, sign='#', preview_data=None, condition=lambda obj: True):
        """
        传入一个可迭代对象iterable， 传入一个条件condition，yield 满足条件的元素。
        在这个过程中，会在控制台打印一个进度条，进度条符号用sign表示。
        如果传入了preview_data（一个PreviewDataInLoop 的实例），会截取一部分元素作为预览数据。
        :param iterable:
        :param sign:
        :param preview_data:
        :param condition:
        """
        self.iterable = iterable
        self.sign = sign
        self.preview_data = preview_data
        self.count = 0
        self.condition = condition

    def __len__(self):
        return len(self.iterable)

    def __iter__(self):
        bar = True
        for i, obj in enumerate(self.iterable, 1):
            if self.condition(obj):
                if isinstance(self.preview_data, PreviewDataInLoop):
                    self.preview_data.append(obj)
                    # self.preview_data.count += 1
                    # if self.preview_data.count <= self.preview_data.limit:
                    #     self.preview_data.preview.append(obj)
                yield obj

            percentage = i * 100 / self.__len__()
            if bar:
                if percentage < 99:
                    writing = self.sign * int(percentage / 2) + '-' * (50 - int(percentage / 2)) + '%d%%\r' % (percentage)
                    sys.stdout.write(writing)
                    sys.stdout.flush()
                else:
                    sys.stdout.write(self.sign * 50 + '100%\r\n')
                    bar = False
        self.count = i


class PreviewDataInLoop(object):
    """
    用于获取for 循环中的预览数据，用法见于WithProgressBar类的注释信息，也可见于以下信息：
        preview_data = PreviewDataInBar()
        for i in preview_data.with_progress_bar(range(1000)):
            time.sleep(0.1)

        ##################################################100%
        print(preview_data.preview)
        [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11,... ...199]

    为什么要在for循环中取一部分预览数据？——这在数据库取数据中有用。
    如何作用于while 循环？没什么特别好的方法。
        preview_data = PreviewDataInBar()
        while some_condition(obj):
            preview_data.append(obj)
    """
    def __init__(self, limit=200):

        self.preview = []
        self.limit = limit
        self.count = 0

    def append(self, obj):
        self.count += 1
        if self.count <= self.limit:
            self.preview.append(obj)

    def with_progress_bar(self, iterable, sign='#', condition=lambda obj: True):
        return WithProgressBar(iterable=iterable, sign=sign, preview_data=self, condition=condition)






from setuptools import setup


setup(
    name='zoran_tools',
    version='0.1.40',
    author='bkiu',
    author_email='chenzhongrun@bonc.com.cn',
    description='自用的一些小工具,包括列出文件夹下的文件, 读写CSV等',
    packages=['zoran_tools'],
)
